#!/usr/local/bin/python
# -*- coding: utf-8 -*-
"""Entity grid module for TRUNAJOD.

In this module, entity grid based features are implemented. On one side,
an entity grid :cite:`barzilay2008modeling` implementation is provided.
We also provide an implementation of the entity graph coherence modeling
:cite:`guinaudeau2013graph`.

.. danger:: These set of features or measurements
   Really depends on the dependency parsing accuracy, which relies on the
   CORPUS the dependency parsed was trained. There is no guarantee that this
   will work with all types of texts. On the other hand, the implementation
   is simple and we do not do any coreference resolution for noun-phrases and
   just rely on simple heuristics.

It is also worth noting, that we consider an entity grid of two-sentence
sequence and the API currently does not provide any hyper-parameter tunning to
change this.
"""

SPACY_UNIVERSAL_NOUN_TAGS = set([u'NOUN', u'PRON', u'PROPN'])

ordered_transitions = [
    u'SS', u'SO', u'SX', u'S-', u'OS', u'OO', u'OX', u'O-', u'XS', u'XO',
    u'XX', u'X-', u'-S', u'-O', u'-X', u'--'
]


def dependency_mapping(dep):
    """Map dependency tag to entity grid tag.

    We consider the notation provided in :cite:`barzilay2008modeling`:

    +-----------+-----------------------------------+
    | EGrid Tag | Dependency Tag                    |
    +===========+===================================+
    | S         | nsub, csubj, csubjpass, dsubjpass |
    +-----------+-----------------------------------+
    | O         | iobj, obj, pobj, dobj             |
    +-----------+-----------------------------------+
    | X         | For any other dependency tag      |
    +-----------+-----------------------------------+

    :param dep: Dependency tag
    :type dep: string
    :return: EGrid tag
    :rtype: string
    """
    if dep in {u'nsubj', u'csubj', u'csubjpass', u'dsubjpass'}:
        return u'S'
    if dep in {u'iobj', u'obj', u'pobj', u'dobj'}:
        return u'O'

    return 'X'


class EntityGrid(object):
    """Entity grid class.

    Class Entity Grid, creates an entity grid from a doc, which is output of
    applying spacy.nlp(text) to a text. Thus, this class depends on spacy
    module. It only supports 2-transitions entity grid.
    """

    def __init__(self, doc):
        """Construct EntityGrid object."""
        # Initialization
        entity_map = dict()
        entity_grid = dict()
        i = 1
        entity_map['s%d' % i] = []
        entity_features = {
            u'SS': 0,
            u'SO': 0,
            u'SX': 0,
            u'S-': 0,
            u'OS': 0,
            u'OO': 0,
            u'OX': 0,
            u'O-': 0,
            u'XS': 0,
            u'XO': 0,
            u'XX': 0,
            u'X-': 0,
            u'-S': 0,
            u'-O': 0,
            u'-X': 0,
            u'--': 0
        }

        # Get number of sentences in the text
        n_sent = len(list(doc.sents))

        # To get coherence measurements we need at least 2 sentences
        if n_sent < 2:
            raise RuntimeError(
                "Entity grid needs at least two sentences, found: {}"
                .format(n_sent))

        # For each sentence, get dependencies and its grammatical role
        for sent in doc.sents:
            for token in sent:
                if token.pos_ in SPACY_UNIVERSAL_NOUN_TAGS:
                    entity_map['s%d' % i].append((token.text.upper(),
                                                  token.dep_))
                    if token.text.upper() not in entity_grid:
                        entity_grid[token.text.upper()] = [u'-'] * n_sent
            i += 1
            entity_map['s%d' % i] = []

        # Last iteration will create an extra element, so I remove it.
        entity_map.pop('s%d' % i)

        # Fill entity grid
        for i in range(n_sent):
            sentence = "s%d" % (i + 1)
            for entity, dep in entity_map[sentence]:
                if entity_grid[entity][i] == u'-':
                    entity_grid[entity][i] = dependency_mapping(dep)
                elif dependency_mapping(dep) == u'S':
                    entity_grid[entity][i] = dependency_mapping(dep)
                elif (dependency_mapping(dep) == u'O'
                      and entity_grid[entity][i] == u'X'):
                    entity_grid[entity][i] = dependency_mapping(dep)

        # Compute feature vector, we consider transitions of length 2
        total_transitions = (n_sent - 1) * len(entity_grid.keys())

        for entity in entity_grid:
            for i in range(n_sent - 1):
                # Transition type found (e.g. S-)
                transition = (
                    entity_grid[entity][i] + entity_grid[entity][i + 1])

                # Adding 1 to transition count
                entity_features[transition] += 1

        for prob in entity_features:
            if total_transitions != 0:
                entity_features[prob] /= float(total_transitions)
            else:
                entity_features[prob] = 0.0

        self.__grid = entity_grid
        self.__n_sent = n_sent
        self.__prob = entity_features

    def get_ss_transitions(self):
        """Get SS transitions.

        :return: Ratio of transitions
        :rtype: float
        """
        return self.__prob[u"SS"]

    def get_so_transitions(self):
        """Get SO transitions.

        :return: Ratio of transitions
        :rtype: float
        """
        return self.__prob[u"SO"]

    def get_sx_transitions(self):
        """Get SX transitions.

        :return: Ratio of transitions
        :rtype: float
        """
        return self.__prob[u"SX"]

    def get_sn_transitions(self):
        """Get S- transitions.

        :return: Ratio of transitions
        :rtype: float
        """
        return self.__prob[u"S-"]

    def get_os_transitions(self):
        """Get OS transitions.

        :return: Ratio of transitions
        :rtype: float
        """
        return self.__prob[u"OS"]

    def get_oo_transitions(self):
        """Get OO transitions.

        :return: Ratio of transitions
        :rtype: float
        """
        return self.__prob[u"OO"]

    def get_ox_transitions(self):
        """Get OX transitions.

        :return: Ratio of transitions
        :rtype: float
        """
        return self.__prob[u"OX"]

    def get_on_transitions(self):
        """Get O- transitions.

        :return: Ratio of transitions
        :rtype: float
        """
        return self.__prob[u"O-"]

    def get_xs_transitions(self):
        """Get XS transitions.

        :return: Ratio of transitions
        :rtype: float
        """
        return self.__prob[u"XS"]

    def get_xo_transitions(self):
        """Get XO transitions.

        :return: Ratio of transitions
        :rtype: float
        """
        return self.__prob[u"XO"]

    def get_xx_transitions(self):
        """Get XX transitions.

        :return: Ratio of transitions
        :rtype: float
        """
        return self.__prob[u"XX"]

    def get_xn_transitions(self):
        """Get X- transitions.

        :return: Ratio of transitions
        :rtype: float
        """
        return self.__prob[u"X-"]

    def get_ns_transitions(self):
        """Get -S transitions.

        :return: Ratio of transitions
        :rtype: float
        """
        return self.__prob[u"-S"]

    def get_no_transitions(self):
        """Get -O transitions.

        :return: Ratio of transitions
        :rtype: float
        """
        return self.__prob[u"-O"]

    def get_nx_transitions(self):
        """Get -X transitions.

        :return: Ratio of transitions
        :rtype: float
        """
        return self.__prob[u"-X"]

    def get_nn_transitions(self):
        """Get -- transitions.

        :return: Ratio of transitions
        :rtype: float
        """
        return self.__prob[u"--"]

    def get_egrid(self):
        """Return obtained entity grid (for debugging purposes).

        :return: entity grid represented as a dict
        :rtype: dict
        """
        return self.__grid

    def get_sentence_count(self):
        """Return sentence count obtained while processing.

        :return: Number of sentences
        :rtype: int
        """
        return self.__n_sent


def weighting_syntactic_role(entity_role):
    """Return weight given an entity grammatical role.

    Weighting scheme for syntactic role of an entity. This uses the heuristic
    from :cite:`guinaudeau2013graph`, which is:

    +-----------+--------+
    | EGrid Tag | Weight |
    +===========+========+
    | S         | 3      |
    +-----------+--------+
    | O         | 2      |
    +-----------+--------+
    | X         | 1      |
    +-----------+--------+
    | dash      | 0      |
    +-----------+--------+

    :param entity_role: Entity grammatical role (S, O, X, -)
    :type entity_role: string
    :return: Role weight
    :rtype: int
    """
    if entity_role == u"S":
        return 3
    elif entity_role == u"O":
        return 2
    elif entity_role == u"X":
        return 1

    return 0


def get_local_coherence(egrid):
    """Get local coherence from entity grid.

    This method gets the coherence value using all the approaches described
    in :cite:`guinaudeau2013graph`. This include:

    * local_coherence_PU
    * local_coherence_PW
    * local_coherence_PACC
    * local_coherence_PU_dist
    * local_coherence_PW_dist
    * local_coherence_PACC_dist

    :param egrid: An EntityGrid object.
    :type egrid: EntityGrid
    :return: Local coherence based on different heuristics
    :rtype: tuple of floats
    """
    n_sent = egrid.get_sentence_count()

    # If entity grid is not valid
    if n_sent < 2:
        return (0.0, 0.0, 0.0, 0.0, 0.0, 0.0)

    PW = [[0] * n_sent for i in range(n_sent)]

    # Weight Matrix for PACC, syntactic information is accounted for by
    # integrating the edges of the bipartite graph
    W = [[0] * n_sent for i in range(n_sent)]

    grid = egrid.get_egrid()
    for entity in grid:
        for i in range(n_sent):
            for j in range(i + 1, n_sent):
                if grid[entity][i] != u"-" and grid[entity][j] != u"-":
                    PW[i][j] += 1
                    W[i][j] += (weighting_syntactic_role(grid[entity][i]) *
                                weighting_syntactic_role(grid[entity][j]))

    PU = [list(map(lambda x: x != 0, PWi)) for PWi in PW]

    local_coherence_PU = 0.0
    local_coherence_PW = 0.0
    local_coherence_PACC = 0.0
    for i in range(n_sent):
        local_coherence_PW += sum(PW[i])
        local_coherence_PU += sum(PU[i])
        local_coherence_PACC += sum(W[i])

    local_coherence_PW /= n_sent
    local_coherence_PU /= n_sent
    local_coherence_PACC /= n_sent

    # Weighting projection graphs
    PU_weighted = list(PU)
    PW_weighted = list(PW)
    PACC_weighted = list(W)
    for i in range(n_sent):
        for j in range(i + 1, n_sent):
            PU_weighted[i][j] = PU[i][j] / float(j - i)
            PW_weighted[i][j] = PW[i][j] / float(j - i)
            PACC_weighted[i][j] = W[i][j] / float(j - i)

    local_coherence_PU_dist = 0.0
    local_coherence_PW_dist = 0.0
    local_coherence_PACC_dist = 0.0
    for i in range(n_sent):
        local_coherence_PW_dist += sum(PW_weighted[i])
        local_coherence_PU_dist += sum(PU_weighted[i])
        local_coherence_PACC_dist += sum(PACC_weighted[i])

    local_coherence_PW_dist /= n_sent
    local_coherence_PU_dist /= n_sent
    local_coherence_PACC_dist /= n_sent
    return (local_coherence_PU, local_coherence_PW, local_coherence_PACC,
            local_coherence_PU_dist, local_coherence_PW_dist,
            local_coherence_PACC_dist)



#!/usr/bin/env python
"""Type Token Ratios module.

Type token ratios (TTR) are a measurement of lexical diversity. They are
defined as the ratio of unique tokens divided by the total number of tokens.
This measurement is bounded between 0 and 1. If there is no repetition in
the text this measurement is 1, and if there is infinite repetition, it will
tend to 0. This measurement is not recommended if analyzing texts of different
lengths, as when the number of tokens increases, the TTR tends flatten.
"""
from TRUNAJOD.utils import is_word


def type_token_ratio(word_list):
    """Return Type Token Ratio of a word list.

    :param word_list: List of words
    :type word_list: List of strings
    :return: TTR of the word list
    :rtype: float
    """
    return len(set(word_list)) / len(word_list)


def lexical_diversity_mtld(doc, ttr_segment=0.72):
    """Compute MTLD lexical diversity in a bi-directional fashion.

    :param doc: Processed text
    :type doc: Spacy Doc
    :return: Bi-directional lexical diversity MTLD
    :rtype: float
    """
    word_list = []
    for token in doc:
        if is_word(token.pos_):
            word_list.append(token.lemma_)
    return (one_side_lexical_diversity_mtld(word_list, ttr_segment) +
            one_side_lexical_diversity_mtld(word_list[::-1], ttr_segment)) / 2


def one_side_lexical_diversity_mtld(doc, ttr_segment=0.72):
    """Lexical diversity per MTLD.

    :param doc: Tokenized text
    :type doc: Spacy Doc
    :param ttr_segment: Threshold for TTR mean computation
    :type ttr_segment: float
    :return: MLTD lexical diversity
    :rtype: float
    """
    factor = 0
    total_words = 0
    non_ttr_segment = 1 - ttr_segment
    word_list = []
    for token in doc:
        word_list.append(token.lower())
        total_words += 1
        ttr = type_token_ratio(word_list)
        if ttr < ttr_segment:
            word_list = []
            factor += 1

    if word_list:
        factor += 1 - (
            type_token_ratio(word_list) - ttr_segment) / non_ttr_segment
        total_words += 1

    return total_words / factor



"""TRUNAJOD lexico semantic norms module.

Lexico-Semantic norms do also require external knowledge to be computed. We compute
the following lexico-semantic variables:

* Arousal
* Concreteness
* Context Availability
* Familiarity
* Imageability
* Valence

We provide two downloadable models of these variables, which come from
:cite:`duchon2013espal` and :cite:`guasch2016spanish`.
"""
from TRUNAJOD.lexicosemantic_norms_espal import LEXICOSEMANTIC_ESPAL
from TRUNAJOD.lexicosemantic_norms_espal import LSNorm
from TRUNAJOD.utils import lemmatize


class LexicoSemanticNorm(object):
    """Create a lexico semantic norm calculator for text.

    This requires a lexico semantic norm dict, with key-value pairs specified
    as ``word -> {"arousal", "concreteness", "context_availability",
    "familiarity", "imageability", "valence"}``. Average over number of
    tokens will be computed. The values are obtained from
    :cite:`guasch2016spanish`.
    """

    def __init__(self, doc, lexico_semantic_norm_dict, lemmatizer=None):
        """Initialize lexico semantic norm object.

        Calculate average over number of tokens given a text.

        :param doc: Text to be processed
        :type doc: Spacy Doc
        :param lexico_semantic_norm_dict: Lexico semantic norms for words
        :type lexico_semantic_norm_dict: dict
        :param lemmatizer: Lemmatizer, defaults to None
        :type lemmatizer: dict, optional
        """
        valence = 0
        arousal = 0
        concreteness = 0
        imageability = 0
        context_availability = 0
        familiarity = 0
        count = 0.0

        for token in doc:
            word = token.text.lower()
            word_lemma = word
            if lemmatizer:
                word_lemma = lemmatize(lemmatizer, word)

            if word in lexico_semantic_norm_dict:
                valence += lexico_semantic_norm_dict[word].get("valence")
                arousal += lexico_semantic_norm_dict[word].get("arousal")
                concreteness += (
                    lexico_semantic_norm_dict[word].get("concreteness"))
                imageability += (
                    lexico_semantic_norm_dict[word].get("imageability"))
                context_availability += (lexico_semantic_norm_dict[word].get(
                    "context_availability"))
                familiarity += (
                    lexico_semantic_norm_dict[word].get("familiarity"))
                count += 1
            elif word_lemma in lexico_semantic_norm_dict:
                word = word_lemma
                valence += lexico_semantic_norm_dict[word].get("valence")
                arousal += lexico_semantic_norm_dict[word].get("arousal")
                concreteness += (
                    lexico_semantic_norm_dict[word].get("concreteness"))
                imageability += (
                    lexico_semantic_norm_dict[word].get("imageability"))
                context_availability += (lexico_semantic_norm_dict[word].get(
                    "context_availability"))
                familiarity += (
                    lexico_semantic_norm_dict[word].get("familiarity"))
                count += 1.0

        self.__valence = valence
        self.__arousal = arousal
        self.__concreteness = concreteness
        self.__imageability = imageability
        self.__context_avilability = context_availability
        self.__familiarity = familiarity
        if count > 0:
            self.__valence /= count
            self.__arousal /= count
            self.__concreteness /= count
            self.__imageability /= count
            self.__context_avilability /= count
            self.__familiarity /= count

    def get_arousal(self):
        """Get arousal.

        :return: Average arousal.
        :rtype: float
        """
        return self.__arousal

    def get_concreteness(self):
        """Get concreteness.

        :return: Average concreteness.
        :rtype: float
        """
        return self.__concreteness

    def get_context_availability(self):
        """Get context_availability.

        :return: Average context_availability.
        :rtype: float
        """
        return self.__context_avilability

    def get_familiarity(self):
        """Get familiarity.

        :return: Average familiarity.
        :rtype: float
        """
        return self.__familiarity

    def get_imageability(self):
        """Get imageability.

        :return: Average imageability.
        :rtype: float
        """
        return self.__imageability

    def get_valence(self):
        """Get valence.

        :return: Average valence.
        :rtype: float
        """
        return self.__valence


def get_conc_imag_familiarity(doc):
    """Get lexico-semantic variables.

    Computes three lexico-semantic variables: Concreteness, Imageability and
    Familiarity. The values are obtained from the EsPal dictionary (Spanish)
    and average of each metric is computed over sentences. To get each metric,
    the best practice is using `LSNorm Enum` defined in
    `lexicosemantic_norms_espal` module. The enums are `CONCRETENESS`,
    `IMAGEABILITY` and `FAMILIARITY`. This implementation uses values of the
    lexico-semantic norms from :cite:`duchon2013espal`.

    :param doc: Tokenized text
    :type doc: Spacy Doc
    :return: Concreteness imageability and familiarity averaged over sentences
    :rtype: List of float
    """
    n_found_tokens = [0, 0, 0]
    lsnorm_total = [0, 0, 0]

    for token in doc:
        if (token.pos_ == "NOUN"):
            lemma = token.lemma_
            if lemma in LEXICOSEMANTIC_ESPAL:
                concreteness, imageability, familiarity =\
                    LEXICOSEMANTIC_ESPAL[lemma.lower()]
                n_found_tokens = [x + 1 for x in n_found_tokens]
                lsnorm_total[LSNorm.CONCRETENESS] += concreteness
                lsnorm_total[LSNorm.IMAGEABILITY] += imageability
                lsnorm_total[LSNorm.FAMILIARITY] += familiarity

    lsnorm_total[LSNorm.CONCRETENESS] /= n_found_tokens[LSNorm.CONCRETENESS]
    lsnorm_total[LSNorm.IMAGEABILITY] /= n_found_tokens[LSNorm.IMAGEABILITY]
    lsnorm_total[LSNorm.FAMILIARITY] /= n_found_tokens[LSNorm.FAMILIARITY]
    return lsnorm_total



"""TRUNAJOD emotions module.

These measurements are based on the lexicon from :cite:`rangel2014creacion`.
TRUNAJOD already comes bundled with this lexicon, in the following module
constant ``TRUNAJOD.spanish_emotion_lexicon.SPANISH_EMOTION_LEXICON``.

.. caution:: Keep in mind
   this module is still under development. Moreover
   only Spanish language is supported by this module as we are directly
   feeding the class with the lexicon.

The following emotions are captured: alegría, enojo, miedo, repulsión,
sorpresa, tristeza.
"""
from TRUNAJOD.spanish_emotion_lexicon import SPANISH_EMOTION_LEXICON
from TRUNAJOD.utils import lemmatize


class Emotions(object):
    """Compute emotions from SPANISH EMOTION LEXICON :cite:`rangel2014creacion`.

    Computes emotions based on Probability Factor of Affective use (PFA) and
    averaging over the total analized words from the text (that are found
    in the lexicon).

    Ideas:

    - How about not using PFA?
    - Averaging over emotions and not the total count?
    """

    def __init__(self, doc, lemmatizer=None):
        """Initialize emotions class.

        Average over number of tokens.

        :param doc: Texto a ser procesado
        :type doc: Spacy Doc
        :param lemmatizer: Lematizador a utilizar, defaults to None
        :type lemmatizer: Python dict, optional
        """
        alegria = 0
        enojo = 0
        miedo = 0
        repulsion = 0
        sorpresa = 0
        tristeza = 0
        count = 0.0

        for token in doc:
            word = token.text.lower()
            word_lemma = word
            if lemmatizer:
                word_lemma = lemmatize(lemmatizer, word)

            if word in SPANISH_EMOTION_LEXICON:
                pfa, emotion = SPANISH_EMOTION_LEXICON[word]
                if emotion == 'Alegría':
                    alegria += pfa
                elif emotion == 'Enojo':
                    enojo += pfa
                elif emotion == 'Miedo':
                    miedo += pfa
                elif emotion == 'Repulsión':
                    repulsion += pfa
                elif emotion == 'Sorpresa':
                    sorpresa += pfa
                else:
                    tristeza += pfa
                count += 1
            elif word_lemma in SPANISH_EMOTION_LEXICON:
                word = word_lemma
                pfa, emotion = SPANISH_EMOTION_LEXICON[word]
                if emotion == 'Alegría':
                    alegria += pfa
                elif emotion == 'Enojo':
                    enojo += pfa
                elif emotion == 'Miedo':
                    miedo += pfa
                elif emotion == 'Repulsión':
                    repulsion += pfa
                elif emotion == 'Sorpresa':
                    sorpresa += pfa
                else:
                    tristeza += pfa
                count += 1.0

        self.__alegria = alegria
        self.__enojo = enojo
        self.__miedo = miedo
        self.__repulsion = repulsion
        self.__sorpresa = sorpresa
        self.__tristeza = tristeza
        if count > 0:
            self.__alegria /= count
            self.__enojo /= count
            self.__miedo /= count
            self.__repulsion /= count
            self.__sorpresa /= count
            self.__tristeza /= count

    def get_alegria(self):
        """Get alegria.

        :return: Average alegria over number of tokens
        :rtype: float
        """
        return self.__alegria

    def get_enojo(self):
        """Get enojo.

        :return: Average enojo over number of tokens
        :rtype: float
        """
        return self.__enojo

    def get_miedo(self):
        """Get miedo.

        :return: Average miedo over number of tokens
        :rtype: float
        """
        return self.__miedo

    def get_repulsion(self):
        """Get repulsion.

        :return: Average repulsion over number of tokens
        :rtype: float
        """
        return self.__repulsion

    def get_sorpresa(self):
        """Get sorpresa.

        :return: Average sorpresa over number of tokens
        :rtype: float
        """
        return self.__sorpresa

    def get_tristeza(self):
        """Get tristeza.

        :return: Average tristeza over number of tokens
        :rtype: float
        """
        return self.__tristeza



"""TRUNAJOD discourse markers module.

This discourse markers module is based on the lexicon described in
:cite:`i2005representing`, and at the point of writing they are under
https://cs.famaf.unc.edu.ar/~laura/shallowdisc4summ/discmar/. TRUNAJOD
comes with this lexicon bundled, and this module has the following set
'constants':

+-------------------------------+---------------------------------------------+
| Type of discourse marker      | Discourse marker                            |
+===============================+=============================================+
| cause                         | dado que, porque, debido a, gracias a,      |
|                               | por si, por, por eso, en conclusión,        |
|                               | así que, como consecuencia, para, para que, |
|                               | por estar razón, por tanto, en efecto       |
+-------------------------------+---------------------------------------------+
| context                       | teniendo en cuenta, después, antes,         |
|                               | originalmente, a condición de, durante,     |
|                               | mientras, a no ser que, cuando, donde,      |
|                               | de acuerdo con, lejos de, tan pronto como,  |
|                               | por el momento, entre, hacia, hasta,        |
|                               | mediante, según, en cualquier caso,         |
|                               | entonces, respecto a, en ese caso, si,      |
|                               | siempre que, sin duda, a la vez             |
+-------------------------------+---------------------------------------------+
| equality                      | en resumen, concretamente, en esencia,      |
|                               | en comparación, en otras palabras,          |
|                               | en particular, es decir, por ejemplo,       |
|                               | precisamente, tal como, por último,         |
|                               | por un lado, por otro lado, a propósito,    |
|                               | no sólo, sino también, en dos palabras,     |
|                               | además, también, aparte, aún es más,        |
|                               | incluso, especialmente, sobretodo           |
+-------------------------------+---------------------------------------------+
| highly polysemic              | como, desde, sobre, antes que nada,         |
|                               | para empezar                                |
+-------------------------------+---------------------------------------------+
| Revision                      | a pesar de, aunque, excepto, pese a,        |
|                               | no obstante, sin embargo, en realidad,      |
|                               | de hecho, al contrario, el hecho es que,    |
|                               | es cierto que, pero', con todo, ahora bien, |
|                               | de todos modos                              |
+-------------------------------+---------------------------------------------+
| Vague meaning closed class    | y, e, ni, o, u, que, con, sin, contra,      |
| words                         | en, a,                                      |
+-------------------------------+---------------------------------------------+

The following module constants are defined as sets ``CAUSE_DISCOURSE_MARKERS``,
``CONTEXT_DISCOURSE_MARKERS``, ``EQUALITY_DISCOURSE_MARKERS``,
``HIGHLY_POLYSEMIC_DISCOURSE_MARKERS``, ``REVISION_DISCOURSE_MARKERS``,
``VAGUE_MEANING_CLOSED_CLASS_WORDS``.
"""
import re

CAUSE_DISCOURSE_MARKERS = {
    'dado que',
    'porque',
    'debido a',
    'gracias a',
    'por si',
    'por',
    'por eso',
    'en conclusión',
    'así que',
    'como consecuencia',
    'para',
    'para que',
    'por esta razón',
    'por tanto',
    'en efecto',
}

CONTEXT_DISCOURSE_MARKERS = {
    'teniendo en cuenta',
    'después',
    'antes',
    'originalmente',
    'a condición de',
    'durante',
    'mientras',
    'a no ser que',
    'cuando',
    'donde',
    'de acuerdo con',
    'lejos de',
    'tan pronto como',
    'por el momento',
    'entre',
    'hacia',
    'hasta',
    'mediante',
    'según',
    'en cualquier caso',
    'entonces',
    'respecto a',
    'en ese caso',
    'si',
    'siempre que',
    'sin duda',
    'a la vez',
}

EQUALITY_DISCOURSE_MARKERS = {
    'en resumen',
    'concretamente',
    'en esencia',
    'en comparación',
    'en otras palabras',
    'en particular',
    'es decir',
    'por ejemplo',
    'precisamente',
    'tal como',
    'por último',
    'por un lado',
    'por otro lado',
    'a propósito',
    'no sólo',
    'sino también',
    'en dos palabras',
    'además',
    'también',
    'aparte',
    'aún es más',
    'incluso',
    'especialmente',
    'sobretodo',
}

HIGHLY_POLYSEMIC_DISCOURSE_MARKERS = {
    'como',
    'desde',
    'sobre',
    'antes que nada',
    'para empezar',
}

REVISION_DISCOURSE_MARKERS = {
    'a pesar de',
    'aunque',
    'excepto',
    'pese a',
    'no obstante',
    'sin embargo',
    'en realidad',
    'de hecho',
    'al contrario',
    'el hecho es que',
    'es cierto que',
    'pero',
    'con todo',
    'ahora bien',
    'de todos modos',
}

VAGUE_MEANING_CLOSED_CLASS_WORDS = {
    'y',
    'e',
    'ni',
    'o',
    'u',
    'que',
    'con',
    'sin',
    'contra',
    'en',
    'a',
}


def find_matches(text, list):
    """Return matches of words in list in a target text.

    Given a text and a list of possible matches (in this module, discourse
    markers list), returns the number of matches found in text. This ignores
    case.

    .. hint:: For non-Spanish users
       You could use this function with your custom list of discourse markers
       in case you need to compute this metric. In that case, the way to call
       the funcion would be: ``find_matches(YOUR_TEXT, ["dm1", "dm2", etc])``

    :param text: Text to be processed
    :type text: string
    :param list: list of discourse markers
    :type list: Python list of strings
    :return: Number of ocurrences
    :rtype: int
    """
    counter = 0
    for w in list:
        results = re.findall(r'\b%s\b' % w, text, re.IGNORECASE)
        counter += len(results)
    return counter


def get_cause_dm_count(text):
    """Count discourse markers associated with cause.

    :param text: The text to be analized
    :type text: Spacy Doc
    :return: Average of revision cause markers over sentences
    :rtype: float
    """
    sentences = [
        find_matches(sent.string.strip(), CAUSE_DISCOURSE_MARKERS)
        for sent in text.sents
    ]
    return sum(sentences) / len(sentences)


def get_closed_class_vague_meaning_count(text):
    """Count words that have vague meaning.

    :param text: The text to be analized
    :type text: Spacy Doc
    :return: Average of vague meaning words over sentences
    :rtype: float
    """
    sentences = [
        find_matches(sent.string.strip(), VAGUE_MEANING_CLOSED_CLASS_WORDS)
        for sent in text.sents
    ]
    return sum(sentences) / len(sentences)


def get_context_dm_count(text):
    """Count discourse markers associated with context.

    :param text: The text to be analized
    :type text: Spacy Doc
    :return: Average of context discourse markers over sentences
    :rtype: float
    """
    sentences = [
        find_matches(sent.string.strip(), CONTEXT_DISCOURSE_MARKERS)
        for sent in text.sents
    ]
    return sum(sentences) / len(sentences)


def get_equality_dm_count(text):
    """Count discourse markers associated with equality.

    :param text: The text to be analized
    :type text: Spacy Doc
    :return: Average of equality discourse markers over sentences
    :rtype: float
    """
    sentences = [
        find_matches(sent.string.strip(), EQUALITY_DISCOURSE_MARKERS)
        for sent in text.sents
    ]
    return sum(sentences) / len(sentences)


def get_polysemic_dm_count(text):
    """Count discourse markers that are highly polysemic.

    :param text: The text to be analized
    :type text: Spacy Doc
    :return: Average of highly polysemic discourse markers over sentences
    :rtype: float
    """
    sentences = [
        find_matches(sent.string.strip(), HIGHLY_POLYSEMIC_DISCOURSE_MARKERS)
        for sent in text.sents
    ]
    return sum(sentences) / len(sentences)


def get_overall_markers(text):  # pragma: no cover
    """Count all types of discourse markers.

    :param text: The text to be analized
    :type text: Spacy Doc
    :return: Average discourse markers over sentences
    :rtype: float
    """
    sentences = []
    sentences += [
        find_matches(sent.string.strip(), VAGUE_MEANING_CLOSED_CLASS_WORDS)
        for sent in text.sents
    ]

    sentences += [
        find_matches(sent.string.strip(), HIGHLY_POLYSEMIC_DISCOURSE_MARKERS)
        for sent in text.sents
    ]

    sentences += [
        find_matches(sent.string.strip(), CONTEXT_DISCOURSE_MARKERS)
        for sent in text.sents
    ]

    sentences += [
        find_matches(sent.string.strip(), EQUALITY_DISCOURSE_MARKERS)
        for sent in text.sents
    ]

    sentences += [
        find_matches(sent.string.strip(), CAUSE_DISCOURSE_MARKERS)
        for sent in text.sents
    ]

    sentences += [
        find_matches(sent.string.strip(), REVISION_DISCOURSE_MARKERS)
        for sent in text.sents
    ]
    return sum(sentences) / len(sentences)


def get_revision_dm_count(text):
    """Count discourse markers associated with revisions.

    :param text: The text to be analized
    :type text: Spacy Doc
    :return: Average of revision discourse markers over sentences
    :rtype: float
    """
    sentences = [
        find_matches(sent.string.strip(), REVISION_DISCOURSE_MARKERS)
        for sent in text.sents
    ]
    return sum(sentences) / len(sentences)



#!/usr/bin/env python
"""Semantic measures TRUNAJOD methods.

The dimensions defined in this module, require external knowledge, for example
synonym overlap measurement requires knowledge from a word onthology, and
semantic measurements require word vectors (word embeddings) obtained from
CORPUS semantics.
"""


def avg_w2v_semantic_similarity(docs, N):
    """Compute average semantic similarity between adjacent sentences.

    This is using word2vec :cite:`mikolov2013word2vec` model based on SPACY
    implementation. The semantic similarity is based on
    :cite:`foltz1998measurement` approach to compute text coherence.

    :param docs: Docs generator provided by SPACY API
    :type docs: Doc Generator
    :param N: Number of sentences
    :type N: int
    :return: Average sentence similarity (cosine)
    :rtype: float
    """
    if N <= 1:  # pragma: no cover
        raise RuntimeError(
            "N of sentences should be > 1, {} was provided".format(N))

    avg_sim = 0
    prev_doc = next(docs)

    # FIXME: Investigate an alternative to this nasty implementation
    done = False
    while not done:
        try:
            curr_doc = next(docs)
            avg_sim += prev_doc.similarity(curr_doc)
            prev_doc = curr_doc
        except StopIteration:
            done = True

    return avg_sim / float(N - 1)


def get_synsets(lemma, synset_dict):
    """Return synonym set given a word lemma.

    The function requires that the synset_dict is passed into it. In our case
    we provide downloadable models from MCR (Multilingual-Central-Repository).
    :cite:`gonzalez2012multilingual`. If the lemma is not found in the
    synset_dict, then this function returns a set with the lemma in it.

    :param lemma: Lemma to be look-up into the synset
    :type lemma: string
    :param synset_dict: key-value pairs, lemma to synset
    :type synset_dict: Python dict
    :return: The set of synonyms of a given lemma
    :rtype: Python set of strings
    """
    return synset_dict.get(lemma, {lemma})


def overlap(lemma_list_group, synset_dict):
    """Compute average overlap in a text.

    Computes semantic synset overlap (synonyms), based on a lemma list group
    and a dictionary containing synsets. Note that the computations are carried
    out dividing by number of text segments considered; matches TAACO
    implementation. For more details about this measurement, refer to
    :cite:`crossley2016tool`

    :param lemma_list_group: List of tokenized and lemmatized sentences
    :type lemma_list_group: List of List of strings
    :param synset_dict: key-value pairs for lemma-synonyms
    :type synset_dict: Python dict
    :return: Average overlap between sentences
    :rtype: float
    """
    if len(lemma_list_group) < 2:  # pragma: no cover
        raise RuntimeError(
            "At least two sentences should be provided, you provided {}"
            .format(lemma_list_group))

    N = len(lemma_list_group)

    overlap_counter = 0
    for i in range(N - 1):
        cur_list = set(lemma_list_group[i])
        next_list = set(lemma_list_group[i + 1])
        for lemma in cur_list:
            for lemma_next in next_list:
                if lemma in get_synsets(lemma_next, synset_dict):
                    overlap_counter += 1

    # I got this from original TAACO code, it seems it is divided by the total
    # segments.
    return overlap_counter / float(N - 1)



"""
Setup script for TRUNAJOD.

This is the setup.py script for TRUNAJOD, to build and package TRUNAJOD.
"""
from distutils.core import setup

setup(
    name='TRUNAJOD',
    version='0.1.0.0',
    license='MIT',
    description="A python lib for readability analyses.",
    long_description=open('README_pypi.rst').read(),
    author='Diego Palma',
    author_email='dipalma@udec.cl',
    url="https://github.com/dpalmasan/TRUNAJOD2.0",
    download_url=(
        "https://github.com/dpalmasan/TRUNAJOD2.0/archive/v0.1.tar.gz"),
    keywords=["NLP", "readability", "entity grid", "linguistics"],
    packages=['TRUNAJOD'],
    package_dir={'': 'src'},
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ])
